--/Example 10

--/ Example of using AGGEGRATE funtions together with the GROUP BY clause to restrict record/row data during data retrieval from a table

--/ Although AGGEGRATE funtions can be used without the GROUP BY clause as in

SELECT 	SUM(freight) AS TotalFreightCosts
FROM 	orders
WHERE 	EmployeeID = 5

--/ and 

SELECT 	MAX(OrderDate) AS LastEverOrderDate
FROM 	orders
WHERE 	EmployeeID = 5

--/ you cannot run queries like this

SELECT 	SUM(freight) AS TotalFreightCosts, CustomerID
FROM 	orders
WHERE 	EmployeeID = 5

--/ and the query has to be corrected with the use of the GROUP BY clause. We can re-write the above query like this

SELECT 	SUM(freight) AS TotalFreightCosts, CustomerID
FROM 	orders
WHERE 	EmployeeID = 5
GROUP BY CustomerID

--/ Multiple columns/fields can be used in the GROUP BY clause so the following query works - not that this is useful!

SELECT 	SUM(freight) AS TotalFreightCosts, CustomerID
FROM 	orders
GROUP BY CustomerID, EmployeeID

--/ More sensibly you would want the EmployeeID in the result set so the above is more usually written as

SELECT 	CustomerID, EmployeeID, SUM(freight) AS TotalFreightCosts
FROM 	orders
GROUP BY CustomerID, EmployeeID

--/ WHAT YOU CANNOT DO IS HAVE MORE COLUMNS/FIELDS IN THE SELECT LIST THAN ARE CONTAINED IN THE GROUP BY CLAUSE
--/ so the following does not work

SELECT 	CustomerID, EmployeeID, SUM(freight) AS TotalFreightCosts
FROM 	orders
GROUP BY CustomerID

--/ note that even columns/fields that have directly related data in them to one of the SELECT fields cannot be included
--/ in the select list if it is not in the group by clause. So although shipaddress is - in this table - almost always
--/ directly linked to the shipname, you cannot add this. But a 'trick' that can be used in this case, when you want some
--/ field in the result set that you don't want to include in the group by clause is to enclose it in some AGGEGATE function
--/ along as this is sensible

SELECT 	CustomerID, ShipName, MAX(ShipAddress) AS ShipAddressAsString, SUM(freight) AS TotalFreightCosts
FROM 	orders
GROUP BY CustomerID, ShipName

--/ This is, in effect, the same as the following and it is personal preference as to which you use

SELECT 	CustomerID, ShipName, ShipAddress AS ShipAddressAsString, SUM(freight) AS TotalFreightCosts
FROM 	orders
GROUP BY CustomerID, ShipName, ShipAddress